package trashplace.btTest1;

public class mwcParser {

	// List of all supported MWC-versions;
	String[]		VersionList		= { "1.7" };

	// Size of param-packet for each supported version.
	int[]			PacketSize		= { 84 };

	int				version;
	dataContainer	mData;
	byte[]			lData;											// local copy of data for faster access.

	// Values from MWC
	public short			accSmooth[]		= { 0, 0, 0 };					// int16_t
	public short			gyroData[]		= { 0, 0, 0 };					// int16_t
	public short			altitudeSmooth	= 0;							// int16_t
	public short			heading			= 0;							// int16_t
	public short			servo[]			= { 0, 0, 0, 0 };				// int16_t
	public short			motor[]			= { 0, 0, 0, 0, 0, 0 };		// int16_t

	// RC-Channels:
	// 0..4: pitch, roll, yaw, Throttle
	// 5..7: aux1, aux2, servo1, servo2
	public short			rcHysteresis[]	= { 0, 0, 0, 0, 0, 0, 0, 0 };	// int16_t

	public boolean			nunchukPresent;								// packed in on byte; bit0 (lsb) 
	public boolean			accPresent;									// ... bit1
	public boolean			baroPresent;									// ... bit2
	public boolean			magPresent;									// ... bit3
	public boolean			accMode;										// packed in one byte; bit0 (lsb)
	public boolean			baroMode;										// ... bit1
	public boolean			magMode;										// ... bit2


	public int				cycleTime;										// uint16_t
	public short			angle[]			= { 0, 0 };					// int16_t
	public int				modelType;										// not explicitly declared, Set by precompiler-directive.
	public String			modelName[]		= { "undef", "Tri", "Quad+",
									"QuadX", "Bi", "Gimbal",
									"Y6", "Hex6", "Flying Wing" , "Y4", "Hex6X"};	// Names f. modelType

	// PID-Arrays: [0]: roll, [1]: pitch, [2]: yaw 
	public int				P8[]			= { 0, 0, 0 };					// uint8_t
	public int				I8[]			= { 0, 0, 0 };					// uint8_t
	public int				D8[]			= { 0, 0, 0 };					// uint8_t
	
	public int				PLEVEL8;										// uint8_t
	public int				ILEVEL8;										// uint8_t
	public int				rcRate8;										// uint8_t
	public int				rcExpo8;										// uint8_t

	public int				rollPitchRate;									// uint8_t
	public int				yawRate;										// uint8_t
	public int				dynThrPID;										// uint8_t
	public int				activateAcc8;									// uint8_t
	public int				activateBaro8;									// uint8_t
	public int				activateMag8;									// uint8_t
	public int				activateCamStab8;								// uint8_t
	public int				activateCamTrig8;								// uint8_t

	/***
	 * 
	 * @param vers
	 *            MWC version 0 = 1.7
	 * @param data
	 *            dataContainer that contains recieved data from BT device
	 */
	public mwcParser(int vers, dataContainer data) {
		super();
		version = vers;
		mData = data;

	}

	public boolean isValid() {
		if (mData.getLength() == PacketSize[version]) return true;
		else return false;
	}

	public int parseData() {
		int result = 0;

		if (isValid()) {
			switch (version) {
			case 0:
				result = parseData_v17(); // 1.7
				break;
			}
		} // if not valid don't do anything. Keep the old values from previous parseData.

		return (result);
	}

	int read16(int pos) {
		return ((lData[pos] & 0xff) + (lData[pos + 1] << 8));
	}

	int read8(int pos) {
		return (lData[pos] & 0xff);
	}

	int parseData_v17() {
		int offs;
		lData = mData.getBuffer(); // work on local copy of data buffer

		// Packet layout:

		// 0; 'A' - start marker

		// 1..6: accSmooth
		offs = 1; 
		accSmooth[0]	= (short)read16(offs); offs+=2;
		accSmooth[1]	= (short)read16(offs); offs+=2;
		accSmooth[2]	= (short)read16(offs); offs+=2;
		
		// 7..12: gyroData
		offs = 7; 
		gyroData[0] = (short)read16(offs); offs+=2;
		gyroData[1] = (short)read16(offs); offs+=2;
		gyroData[2] = (short)read16(offs); offs+=2;
		
		// 13: AltitudeSmooth
		offs = 13; altitudeSmooth	= (short)read16(offs);
		
		// 15: heading
		offs = 15; heading = (short)read16(offs);

		// 17..23: Servo Values
		offs=17; 
		servo[0] = (short)read16(offs); offs+=2;
		servo[1] = (short)read16(offs); offs+=2; 
		servo[2] = (short)read16(offs); offs+=2;
		servo[3] = (short)read16(offs); offs+=2;
		
		// 25..: Motor values 
		offs=25; 
		motor[0] = (short)read16(offs); offs+=2; 
		motor[1] = (short)read16(offs);  offs+=2;
		motor[2] = (short)read16(offs);  offs+=2;
		motor[3] = (short)read16(offs);  offs+=2;
		motor[4] = (short)read16(offs);  offs+=2;
		motor[5] = (short)read16(offs);  offs+=2;

		// 36..: RC Remote Channel values
		offs=37; 
		rcHysteresis[1] = (short)read16(offs);offs+=2; 
		rcHysteresis[0] = (short)read16(offs); offs+=2;
		rcHysteresis[2] = (short)read16(offs); offs+=2;
		rcHysteresis[3] = (short)read16(offs); offs+=2;
		rcHysteresis[4] = (short)read16(offs); offs+=2;
		rcHysteresis[5] = (short)read16(offs); offs+=2;
		rcHysteresis[6] = (short)read16(offs); offs+=2;
		rcHysteresis[7] = (short)read16(offs); offs+=2;

		// 53: Presence-Flags f. NK, ACC, Baro, Mag
		offs = 53;
		if ( (lData[offs] & 0x01) > 0 ) nunchukPresent = true; else nunchukPresent = false;  
		if ( (lData[offs] & 0x02) > 0 ) accPresent = true; else accPresent = false;  
		if ( (lData[offs] & 0x04) > 0 ) baroPresent = true; else baroPresent = false;  
		if ( (lData[offs] & 0x08) > 0 ) magPresent = true; else magPresent = false;  

		// 54: Mode-Flags f. ACC, Baro, Mag
		offs = 54;
		if ( (lData[offs] & 0x01) > 0 ) accMode = true; else accMode = false;  
		if ( (lData[offs] & 0x02) > 0 ) baroMode = true; else baroMode = false;  
		if ( (lData[offs] & 0x04) > 0 ) magMode = true; else magMode = false;  

		// 55: cycleTime
		offs=55;cycleTime=read16(offs);

		// 57: Angles
		offs=57; angle[0] = (short)read16(offs);
		offs=59; angle[1] = (short)read16(offs);
		
		// 61: Model Type
		offs=61; modelType = read8(offs); 
		
		// 62..: P8/I8/D8
		offs=62; 
		// roll
		P8[0] = read8(offs); offs++;
		I8[0] = read8(offs); offs++;
		D8[0] = read8(offs); offs++;
		// pitch
		P8[1] = read8(offs); offs++;
		I8[1] = read8(offs); offs++;
		D8[1] = read8(offs); offs++;
		// yaw
		P8[2] = read8(offs); offs++;
		I8[2] = read8(offs); offs++;
		D8[2] = read8(offs); offs++;
		
		// 71:
		offs=71; 
		PLEVEL8=read8(offs); offs++;
		ILEVEL8=read8(offs); offs++;
		rcRate8=read8(offs); offs++;
		rcExpo8=read8(offs); offs++;
		rollPitchRate=read8(offs); offs++;
		yawRate=read8(offs); offs++;
		dynThrPID=read8(offs); offs++;
		activateAcc8=read8(offs); offs++;
		activateBaro8=read8(offs); offs++;
		activateMag8=read8(offs); offs++;
		activateCamStab8=read8(offs); offs++;
		activateCamTrig8=read8(offs); offs++;
	
		// 83: 'A' - end marker
		
		return 0;
	}
}
